@@ -17,7 +17,7 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-#ident "$Id: Videotex.h,v 1.6 1996/10/31 12:59:14 pierre Exp $"
+#ident "$Id: Videotex.h,v 1.7 1998/10/02 14:50:38 pierre Exp $"
 
 #ifndef _XtVideotex_h
 #define _XtVideotex_h
@@ -107,6 +107,7 @@ extern void videotexDecode(Widget w, int c);
 extern Pixmap videotexPixmapEcranSauve (Widget w);
 extern void videotexSauveEcran (Widget w);
 extern void videotexConversionAscii (Widget w, FILE* fp);
+extern void videotexDumpScreen (Widget w, FILE* fp);
 #else
 extern void videotexDecode ();
 extern void videotexInit ();
@@ -115,6 +116,7 @@ extern void videotexSauveEcran ();
 extern void videotexAfficheTelephone ();
 extern void videotexEffaceTelephone ();
 extern void videotexConversionAscii ();
+extern void videotexDumpScreen ();
 #endif /* __STDC__ */
 
 #endif /* _XtVideotex_h */
