@@ -17,7 +17,7 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-static char rcsid[] = "$Id: xaw.c,v 1.18 1997/07/03 08:23:05 pierre Exp $";
+static char rcsid[] = "$Id: xaw.c,v 1.19 1998/10/02 15:07:11 pierre Exp $";
 
 /*
  * Couche Toolkit ATHENA
@@ -944,6 +944,7 @@ XtPointer call_data;
 	XtRemoveAllCallbacks (ok_boite_saisie, XtNcallback);
 	XtPopdown(popup_saisie);
 	strcpy (nom_fichier_charge, XawDialogGetValueString(boite_saisie));
+	debut_lecteur ();
 	Charge ();
 
 	break;
@@ -1018,6 +1019,7 @@ XtPointer garbage;		/* call_data */
 	    fclose (fp);
 	    strcpy (nom_fichier_charge, name);
 	    XtFree (name);
+	    debut_lecteur ();
 	    Charge ();
 	}
 #endif /* NO_SEL_FILE */
@@ -1128,7 +1130,9 @@ XtPointer garbage;
 #endif
 
     init_xtel ();
-    connexion_service (definition_services[numero_choisi].nom_uucp);
+
+    sprintf (buf, "\001%s", definition_services[numero_choisi].nom_service);
+    connexion_service (buf);
 }
 
 /* ARGSUSED */
