@@ -17,7 +17,7 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-static char rcsid[] = "$Id: xm.c,v 1.13 1997/07/03 08:23:24 pierre Exp $";
+static char rcsid[] = "$Id: xm.c,v 1.14 1998/10/02 15:07:42 pierre Exp $";
 
 /*
  * Couche Toolkit OSF-MOTIF
@@ -711,6 +711,7 @@ XtPointer call_data;
     else {
 	strcpy (nom_fichier_charge, XmTextGetString(text_w));
 	XtDestroyWidget (selecteur_fichier);
+	debut_lecteur ();
 	Charge ();
     }
 }
@@ -776,7 +777,8 @@ XtPointer call_data;
 #endif
 
     init_xtel ();
-    connexion_service (definition_services[numero_choisi].nom_uucp);
+    sprintf (buf, "\001%s", definition_services[numero_choisi].nom_service);
+    connexion_service (buf);
 }
 
 /*
