@@ -17,7 +17,7 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-static char rcsid[] = "$Id: xtel.c,v 1.16 1997/07/03 08:24:47 pierre Exp $";
+static char rcsid[] = "$Id: xtel.c,v 1.17 1998/10/02 15:08:25 pierre Exp $";
 
 /*
  * programme principal 
@@ -47,7 +47,7 @@ static Cursor teleph_cursor;
 static Pixmap pixmap_icone = None;
 static Boolean flag_execution = False;
 static char *procedure_executee = NULL;
-static char *copyright_string = "(C)1991-97 LECTRA SYSTEMES & Pierre Ficheux";
+static char *copyright_string = "(C)1991-98 LECTRA SYSTEMES & Pierre Ficheux";
 static XColor black, white;
 
 /* liste des noms des boutons MINITEL */
@@ -119,6 +119,8 @@ static Boolean defaultFalse = False;
 #define XtNstring8			"string8"
 #define XtNstring9			"string9"
 #define XtNerreur1			"erreur1"
+#define XtNerreur2			"erreur2"
+#define XtNerreur3			"erreur3"
 
 static XtResource ressources[] = {
     {
@@ -254,7 +256,22 @@ static XtResource ressources[] = {
 	XtRString, sizeof(String),
 	XtOffsetOf(ressources_xtel, erreur[0]),
 	XtRString, (XtPointer) "Erreur 0"
+	},
+    {
+	XtNerreur2,
+	XtCString,
+	XtRString, sizeof(String),
+	XtOffsetOf(ressources_xtel, erreur[1]),
+	XtRString, (XtPointer) "Erreur 1"
+	},
+    {
+	XtNerreur3,
+	XtCString,
+	XtRString, sizeof(String),
+	XtOffsetOf(ressources_xtel, erreur[2]),
+	XtRString, (XtPointer) "Erreur 2"
 	}
+
   };
 
 /*
