@@ -17,7 +17,7 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-#ident "$Id: xtel.h,v 1.7 1997/07/03 08:25:00 pierre Exp $"
+#ident "$Id: xtel.h,v 1.8 1998/10/02 15:09:21 pierre Exp $"
 
 #ifndef _xtel_h
 #define _xtel_h
@@ -82,7 +82,7 @@ typedef struct _ressources_xtel {
     Boolean lowMemory;
     Boolean privateColormap;
     String string[9];
-    String erreur[1];
+    String erreur[3];
 } ressources_xtel;
 
 /* nb de procedures */
@@ -96,6 +96,8 @@ typedef struct _ressources_xtel {
 #define P_WRI		5
 #define P_HUP		6
 #define P_EXI		7
+#define P_ADMP		8
+#define P_PDMP		9
 
 struct keyword {
     char *mnemonic;
