@@ -17,7 +17,7 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
-static char rcsid[] = "$Id: xteld.c,v 1.27 1997/07/11 10:56:40 pierre Exp $";
+static char rcsid[] = "$Id: xteld.c,v 1.29 1998/10/02 15:13:39 pierre Exp $";
 
 /*
  * Demon XTELD (communication avec le MODEM)
@@ -36,10 +36,6 @@ static char rcsid[] = "$Id: xteld.c,v 1.27 1997/07/11 10:56:40 pierre Exp $";
  *	- traitement du Minitel 2
  *	- utilisation de syslog
  *
- *   Vincent Gillet
- *
- *	- dialogue avec client Window$ (option -w)
- *
  *   Eric Delaunay
  *
  *	- support IAN
@@ -52,6 +48,7 @@ static char rcsid[] = "$Id: xteld.c,v 1.27 1997/07/11 10:56:40 pierre Exp $";
 #include <ctype.h>
 #include "demon.h"
 #include "globald.h"
+#include "patchlevel.h"
 
 #include <errno.h>
 #include <sys/types.h>
@@ -60,9 +57,6 @@ static char rcsid[] = "$Id: xteld.c,v 1.27 1997/07/11 10:56:40 pierre Exp $";
 #include <time.h>
 #include <signal.h>
 #include <string.h>
-#ifdef USE_SYSLOG
-#include <syslog.h>
-#endif /* USE_SYSLOG */
 
 #ifdef NO_TERMIO
 #include <sgtty.h>
@@ -75,8 +69,9 @@ static char rcsid[] = "$Id: xteld.c,v 1.27 1997/07/11 10:56:40 pierre Exp $";
 #endif /* USE_TERMIOS */
 #endif /* NO_TERMIO */
 
-#ifdef NO_NETWORK
 #include <sys/socket.h>
+
+#ifdef NO_NETWORK
 #include <sys/un.h>
 #include <fcntl.h>
 #ifdef sun
@@ -91,27 +86,13 @@ static int sock_service;
 #else
 
 #include <netinet/in.h>
+#include <netdb.h>
+
 #define XTELD_INPUT	0
 #define XTELD_OUTPUT	1
 
 #endif /* NO_NETWORK */
 
-/* Tableau pour Emulateur sous Windaube */
-static char tablo_windaube[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,  36,
-			 37, 102, 103,  36,  37, 102, 103,  60,
-			 61, 126, 127,  60,  61, 126, 127,  60,
-			 61, 126, 127,  60,  61, 126, 127,  60,
-			 61, 126, 127,  60,  61, 126, 127,  98,
-			 99,  98,  99, 102, 103, 102, 103, 122,
-			123, 122, 123, 126, 127, 126, 127, 122,
-			123, 122, 123, 126, 127, 126, 127, 122,
-			123, 122, 123, 126, 127, 126, 127, 102,
-			103, 102, 103, 102, 103, 102, 103, 126,
-			127, 126, 127, 126, 127, 126, 127, 126,
-			127, 126, 127, 126, 127, 126, 127, 126,
-			127, 126, 127, 126, 127, 126};
-
 static int fin_fils;
 static int pid_fils;
 static int nb_services;
@@ -121,40 +102,47 @@ static time_t t_connexion;
 static char buf[256], buf1[256], service[256], utilisateur[256];
 static char flag_connexion;
 static char parite;
-static Boolean flag_serveur_local, flag_crlf, flag_windaube;
+static Boolean flag_serveur_local, flag_crlf;
+static char type_client = CLIENT_XTEL;
+static int current_page = 0;
+static char erase_needed = 1;
 static int tuyau_in[2], tuyau_out[2];
 static int temps_maxi;
 static int detecte_ian;
 static int saisie_active;
 static char fin_connexion = 0;
+static struct definition_service services_hyper[MAX_SERVICES];
+static int nb_services_hyper;
+static char *peer_name;
+static int first_col;
 
-/* Syslog or not syslog ? */
-#ifdef USE_SYSLOG
-void log_debug (fmt, p1, p2, p3, p4, p5, p6, p7)
-char *fmt;
-int  p1, p2, p3, p4, p5, p6, p7;
-{
-    char msg[256];
+/* 
+ * Table des caracteres accentues avec leurs equivalents ou approximations
+ * en Videotex 
+ */
+static char *special_char_table[] = {
+  "\031AA", "\031BA", "\031CA", "A", "\031HA", "\0310A", "A",
+  "\031KC",
+  "\031AE", "\031BE", "\031CE", "\031HE",
+  "\031AI", "\031BI", "\031CI", "\031HI",
+  "D", "N",
+  "\031AO", "\031BO", "\031CO", "O", "\031HO",
+  "X", "O",
+  "\031AU", "\031BU", "\031CU", "\031HU",
+  "Y", "P", "B",
+  "\031Aa", "\031Ba", "\031Ca", "a", "\031Ha", "\0310a", "a",
+  "\031Kc",
+  "\031Ae", "\031Be", "\031Ce", "\031He",
+  "\031Ai", "\031Bi", "\031Ci", "\031Hi",
+  "d", "n",
+  "\031Ao", "\031Bo", "\031Co", "o", "\031Ho",
+  "-", "o",
+  "\031Au", "\031Bu", "\031Cu", "\031Hu",
+  "\031By", "p", "\031Hy"
+};
 
-    sprintf (msg, fmt, p1, p2, p3, p4, p5, p6, p7);
-    syslog(LOG_DEBUG, msg);
-}
-
-void log_err (s)
-char *s;
-{
-    syslog(LOG_ERR, s);
-}
-#else
-void log_debug (fmt, p1, p2, p3, p4, p5, p6, p7)
-char *fmt;
-int  p1, p2, p3, p4, p5, p6, p7;
-{
-    fprintf (fp_console, "xteld[%d] ", getpid());
-    fprintf (fp_console, fmt, p1, p2, p3, p4, p5, p6, p7);
-    fprintf (fp_console, "\n\r");
-}
-#endif /* USE_SYSLOG */
+static void display_control_line (char *);
+static void erase_control_line (int);
 
 static void demande_fin_fils()
 {
@@ -165,10 +153,19 @@ static void demande_fin_fils()
     fin_fils = 1;
 }
 
+static void sigsegv ()
+{
+  log_err ("Got SIGSEGV");
+  exit (1);
+}
+
+
 /*
  * Transmet une erreur a XTEL 
  *	- chaine de caractere terminee par 0
  *	- errno associe
+ *
+ * En mode HyperTerminal, on affiche l'erreur sur la ligne 0
  */
 void erreur_a_xtel (s, code_erreur)
 char *s;
@@ -176,12 +173,33 @@ int code_erreur;
 {
     char e = code_erreur;
 
-    write (XTELD_OUTPUT, CHAINE_REPONSE_DEBUT_ERREUR, 1);
-    write (XTELD_OUTPUT, s, strlen(s));
-    write (XTELD_OUTPUT, "\0000", 1); 
-    write (XTELD_OUTPUT, &e, 1);
-    write (XTELD_OUTPUT, CHAINE_REPONSE_FIN_ERREUR, 1);
+    if (type_client == CLIENT_XTEL) {
+      write (XTELD_OUTPUT, CHAINE_REPONSE_DEBUT_ERREUR, 1);
+      write (XTELD_OUTPUT, s, strlen(s));
+      write (XTELD_OUTPUT, "\0000", 1); 
+      write (XTELD_OUTPUT, &e, 1);
+      write (XTELD_OUTPUT, CHAINE_REPONSE_FIN_ERREUR, 1);
+    }
+    else if (type_client == CLIENT_W_HYPER) {
+      if (e)
+	sprintf (buf, "%s: %s", s, sys_errlist[errno]);
+      else {
+	if (*s == '[')
+	  strcpy (buf, s + 3);
+	else
+	  strcpy (buf, s);
+      }
+		
+      display_control_line (buf);
+    }
 }
+
+static void usage (char *s)
+{
+  fprintf (stderr, "Usage: %s [-H] [-v]\n", s);
+  exit (1);
+}
+
 /* 
  * Teste si l'utilisateur courant a acces au service demande
  *
@@ -189,10 +207,11 @@ int code_erreur;
  *	0 sinon
  */
 int
-service_autorise (indice_service, maxtime) 
+service_autorise (indice_service, maxtime)
 int indice_service, *maxtime;
-{    char *pt, *pt1, *auth = definition_services[indice_service].autorisations;
-     char autorisations[2048];
+{    
+     char *pt, *pt1, *auth = definition_services[indice_service].autorisations;
+     char autorisations[2048], *host;
      int default_maxtime = 0;
 
      *maxtime = 0;
@@ -225,13 +244,48 @@ int indice_service, *maxtime;
 	   *maxtime = atoi (pt1+1);
 	 }
 
-	 if (strcmp (pt, utilisateur) == 0) {
-           return(1);
+	 /* 
+	  * Test d'autorisation 
+	  * 
+	  * Le nom peut etre sous la forme 'user@hostname' afin d'autoriser un 
+	  * utilisateur donne d'une machine donnee (@hostname autorise tous les 
+	  * utilisateurs de hostname).
+	  *
+	  * Si le hostname n'est pas present, tous les utilisateurs sont
+	  * autorises (compatible avec les versions precedentes de xteld).
+	  * Dans le cas de Windaube, le nom de l' utilisateur est ignore pour
+	  * l'instant.
+	  */
+
+	 
+#ifdef DEBUG_XTELD
+	 log_debug ("peer= %s user= %s pt= %s", peer_name, utilisateur, pt);
+#endif
+
+	 if (*pt == '@') {
+	   if (!strcmp (pt+1, peer_name))
+	     return 1;
+	 }
+	 else {
+	   /* Sinon, test par user@host */
+	   host = strchr (pt, '@');
+	   if (host) {
+	     *host = 0;
+	     host++;
+	     
+	     if (!strcmp (host, peer_name) && (!strcmp (pt, utilisateur) || type_client != CLIENT_XTEL))
+	       return 1;
+	   }
+	   else {
+	     if (!strcmp (pt, utilisateur))
+	       return 1;
+	   }
 	 }
        }
 
        pt = strtok(NULL, ":");
      }
+     
      return(0);
 }
 
@@ -366,14 +420,18 @@ char c;
     return NULL;
 }
 
-/* Trouve le device associe a un numero de telephone */
-static char *find_device (service)
+/* 
+ * Trouve le device associe a un numero de telephone 
+ * ou un nom de service 
+ */
+static char *find_device (service, index)
 char *service;
+int *index;
 {
   register int i, idev;
-  char *numero_direct;
+  char *numero_direct, *pt1, *pt2;
 
-  /* On precise le device "a la main" (ex: 1234,/dev/cua0) */
+  /* On precise le device "a la main" (ex: 3616,le_v23) */
   numero_direct = strchr (service, ',');
 	
   /* Nom du device associe au service */
@@ -383,18 +441,46 @@ char *service;
   }
   else {
     for (i = 0 ; i != nb_services ; i++) {
-      if (!strcmp (definition_services[i].nom_service, "Direct"))
-	  idev = i;
-      if (!strcmp (definition_services[i].nom_uucp, service)) {
-	idev = i;
-	break;
+      /* Si on n'a pas precise le device pour un appel direct, on utilise le
+	 premier trouve */
+      if (!strcmp (definition_services[i].nom_service, "Direct")) {
+	  break;
+      }
+      else {
+	/* Nouvelle interface: xtel envoit le nom du service et non plus le
+	   numero de telephone. Cela permet d'utiliser plusieurs modems pour le
+	   meme numero de telephone */
+	if (*service == '\001') {
+	  pt1 = definition_services[i].nom_service;
+	  pt2 = service + 1;
+	}
+	else {
+	  pt1 = definition_services[i].nom_uucp;
+	  pt2 = service;
+	}
+
+#ifdef DEBUG_XTELD
+	log_debug ("pt1= %s pt2= %s", pt1, pt2);
+#endif
+
+	if (!strcmp (pt1, pt2)) {
+	  break;
+	}
       }
     }
 
+    /* Service non trouve */
+    if (i == nb_services)
+      return NULL;
+
+    idev = i;
+
     (void)service_autorise (idev, &temps_maxi);
 #ifdef DEBUG_XTELD
-      log_debug ("find_device: %s tmax %d", definition_services[idev].device, temps_maxi);
+    log_debug ("find_device: idev= %d %s tmax %d", idev, definition_services[idev].device, temps_maxi);
 #endif
+    *index = idev;
+
     return (definition_services[idev].device);
   }
 }
@@ -406,24 +492,43 @@ void appel_service (service_teletel)
 char *service_teletel;
 {
     char *device_associe = NULL, *option = NULL;
-    char *code_service = NULL;
+    char *code_teletel = NULL, *code_service = NULL;
     FILE *fplog;
     register int i;
+    int idef;
+
+#ifdef DEBUG_XTELD
+    log_debug ("appel_service: service_teletel= %s", service_teletel);
+#endif
 
     detecte_ian = 0;
     saisie_active = 0;
 
-#ifdef DEBUG_XTELD
-    log_debug ("connexion au service %s utilisateur %s", service_teletel, utilisateur);
-#endif
-
     signal (SIGTERM, demande_fin_fils);
 
     /* Calcul du device associe */
-    device_associe = find_device (service_teletel);
+    device_associe = find_device (service_teletel, &idef);
+
+    /* Pas de device ==> erreur */
+    if (!device_associe) {
+      erreur_a_xtel ("[2] Pas de device correspondant !", 0);
+      return;
+    }
+
+    /* Nouvelle interface ? */
+    if (*service_teletel == '\001') {
+      if (idef >= 0)
+	code_teletel = definition_services[idef].nom_uucp;
+    }
+    else
+      code_teletel = service_teletel;
+
+#ifdef DEBUG_XTELD
+    log_debug ("appel_service: service_teletel= %s code_teletel= %s utilisateur= %s", service_teletel, code_teletel, utilisateur);
+#endif
 
     /* Service local par "pipe" */
-    if (device_associe != NULL && strncmp (device_associe, "@pipe", 5) == 0) {
+    if (strncmp (device_associe, "@pipe", 5) == 0) {
 	flag_serveur_local = True;
 
 	/* Options du service local (pour l'instant "crlf") */
@@ -431,21 +536,13 @@ char *service_teletel;
 	    flag_crlf = True;
 
 	if (pipe (tuyau_in) < 0) {
-#ifdef USE_SYSLOG
 	    log_err ("pipe: tuyau_in: %m");
-#else
-	    fprintf (fp_console, "pipe: tuyau_in: %s\n", sys_errlist[errno]);
-#endif /* USE_SYSLOG */
-	    exit (0);
+	    exit (1);
 	}
 
 	if (pipe (tuyau_out) < 0) {
-#ifdef USE_SYSLOG
 	    log_err ("pipe: tuyau_out: %m");
-#else
-	    fprintf (fp_console, "pipe: tuyau_out: %s\n", sys_errlist[errno]);
-#endif /* USE_SYSLOG */
-	    exit (0);
+	    exit (1);
 	}
 
 	if (!fork()) { /* fiston */
@@ -458,19 +555,15 @@ char *service_teletel;
 	    close (tuyau_in[1]);
 
 	    /* On recupere l'option eventuelle */
-	    option = separe_chaine (service_teletel, ' ');
+	    option = separe_chaine (code_teletel, ' ');
 
 #ifdef DEBUG_XTELD
-	    log_debug ("service_teletel %s option -%s-", service_teletel, option);
+	    log_debug ("code_teletel %s option -%s-", code_teletel, option);
 #endif
 
-	    if (execlp (service_teletel, service_teletel, option, NULL) < 0) {
-#ifdef USE_SYSLOG
+	    if (execlp (code_teletel, code_teletel, option, NULL) < 0) {
 		log_err ("execlp: %m");
-#else
-		fprintf (fp_console, "execlp: %s: %s\n", service_teletel, sys_errlist[errno]);
-#endif /* USE_SYSLOG */
-		exit (0);
+		exit (1);
 	    }
 	}
 	else {
@@ -483,14 +576,17 @@ char *service_teletel;
     else 
 #ifndef NO_NETWORK
       /* Connexion TCP */
-      if (device_associe != NULL && strncmp (device_associe, "@tcp", 4) == 0) {
+      if (strncmp (device_associe, "@tcp", 4) == 0) {
+	/* Valide la saisie */
+	saisie_active = 1;
+	
 	/* Recupere les parametres */
-	option = separe_chaine (service_teletel, ':');
+	option = separe_chaine (code_teletel, ':');
 	if (is_number (option)) {
-	  fd_modem = c_clientbyport (service_teletel, atoi(option));
+	  fd_modem = c_clientbyport (code_teletel, atoi(option));
 	}
 	else {
-	  fd_modem = c_clientbyname (service_teletel, option);
+	  fd_modem = c_clientbyname (code_teletel, option);
 	}
       }
     else
@@ -499,30 +595,31 @@ char *service_teletel;
 	/*
 	 * spare tlphone et code service (nnnn:code)
 	 */
-	char *telno = strdup(service_teletel);
-	code_service = strchr( telno, ':' );
+	char *telno = strdup (code_teletel);
+	code_service = strchr (telno, ':');
 	if (code_service) {
 	    *code_service = 0;
-	    code_service = strchr( service_teletel, ':' ) + 1;
+	    code_service = strchr (code_teletel, ':') + 1;
 	    detecte_ian = IAN_DE_GARDE;	/* attente capture code d'entre sur le service */
-	    ian_init( telno );
-	    signal( SIGUSR1, active_saisie );
+	    ian_init (telno);
+	    signal (SIGUSR1, active_saisie);
 	}
-	else {
+	else
 	    saisie_active = 1;
-	}
+
 	/*
 	 * compose le numro
 	 */
 #ifdef DEBUG_XTELD
-	log_debug( "service appel : %s [%s]", telno, code_service?code_service:"" );
+	log_debug( "service appele : %s [%s]", telno, code_service ? code_service:"" );
 #endif
-	if ((fd_modem = mydial(telno, device_associe)) < 0) {
+	if ((fd_modem = mydial (telno, device_associe)) < 0) {
 #ifdef DEBUG_XTELD
 	    log_debug ("meurt (erreur)");
 #endif
 	    exit (1);
 	}
+
 	free(telno);
     }
 
@@ -538,27 +635,30 @@ char *service_teletel;
 #endif
 	
     flag_connexion = 1;	/* On est connecte ! */
-	
-    /* Envoi du temps maxi de connexion */
-    if (temps_maxi) {
-      char l;
 
-      write (XTELD_OUTPUT, CHAINE_TEMPS_MAXI, 1);
-      sprintf (buf, "%d", temps_maxi);
-      l = strlen(buf);
-      write (XTELD_OUTPUT, &l, 1);
-      write (XTELD_OUTPUT, buf, l);
+    if (type_client == CLIENT_XTEL) {
+      /* Envoi du temps maxi de connexion */
+      if (temps_maxi) {
+	char l;
+
+	write (XTELD_OUTPUT, CHAINE_TEMPS_MAXI, 1);
+	sprintf (buf, "%d", temps_maxi);
+	l = strlen(buf);
+	write (XTELD_OUTPUT, &l, 1);
+	write (XTELD_OUTPUT, buf, l);
+      }
+
+      /* signale la connexion a XTEL */
+      write (XTELD_OUTPUT, CHAINE_REPONSE_CONNEXION, 1);
     }
-
-    /* signale la connexion a XTEL */
-    write (XTELD_OUTPUT, CHAINE_REPONSE_CONNEXION, 1);
+    else
+      display_control_line ("CONNECTE AU SERVICE");
 
     /*
      * On cree un processus qui lit le modem et ecrit sur le reseau.
      * On tue ce processus au bout de DELAI_DECONNEXION secondes sans activite 
      * (ce qui fait raccrocher le MODEM)
      */
-    
     if ((pid_fils = fork()) == 0) { /* fils */
 	int etat = 0;
 	int ignore = 0;
@@ -616,11 +716,11 @@ char *service_teletel;
 		int ian;
 
 		for( i = 0 ; i < size ; i++ )
-		    if (ian = ian_valide( detecte_ian, buf[i] )) {
+		    if (ian = ian_valide (detecte_ian, buf[i] )) {
 			switch (detecte_ian) {
 			case IAN_DE_GARDE:
 #ifdef DEBUG_XTELD
-			    log_debug( "Emission code du service : %s + ENVOI", code_service );
+			    log_debug ("Emission code du service : %s + ENVOI", code_service );
 #endif
 			    /* ecrit le code du service */
 			    write ((flag_serveur_local ? tuyau_out[1] : fd_modem),
@@ -630,6 +730,7 @@ char *service_teletel;
 				    "\023A", 2 );
 			    detecte_ian = IAN_D_ENTREE;  /* autorise le service */
 			    break;
+
 			case IAN_D_ENTREE:
 			    if (ian == 1) {
 #ifdef DEBUG_XTELD
@@ -650,6 +751,7 @@ char *service_teletel;
 				fin_fils = 1;	/* chec */
 			    }
 			    break;
+
 			case IAN_DE_FIN:
 #ifdef DEBUG_XTELD
 			    log_debug( "retour  la page Teletel" );
@@ -681,6 +783,7 @@ char *service_teletel;
 			    ignore = 2;
 			}
 			break;
+
 		      case 1:
 			if (buf[i] == 0x53)
 			    /* Sequence SEP 53 reconnue */
@@ -732,6 +835,7 @@ char *service_teletel;
 	chmod (buf, 0644);
 	if (!flag_serveur_local) 
 	    fprintf (fplog, "LIGNE = %s\n", definition_lignes[numero_ligne].nom);
+	fprintf (fplog, "CLIENT = %s [%s]\n", peer_name, (type_client == CLIENT_XTEL ? "Xtel" : "HyperTerminal"));
 	fprintf (fplog, "PROCESSUS = %d,%d\n", getpid(), pid_fils);
 	fprintf (fplog, "SERVICE = %s\n", service);
 	fclose (fplog);
@@ -777,133 +881,327 @@ char *ch;
     return 0;
 
 err:
-#ifdef USE_SYSLOG
     log_err ("read: %m");
-#else
-    fprintf (fp_console, "read: %s\n", sys_errlist[errno]);
-#endif /* USE_SYSLOG */
     return -1;
 }
 
+
+static void trop_tard ()
+{
+    log_err ("Pas de reponse du client.\n");
+    exit (1);
+}
+
+static char *get_peer_name ()
+{
+  struct sockaddr_in client;
+  int len;
+
+  /* Adresse du client */
+  len = sizeof (client);
+  if (getpeername (XTELD_INPUT, (struct sockaddr *) &client, &len) < 0) {
+    log_err ("getpeername: %m");
+  }
+  else {
+    struct hostent *h;
+
+    if ((h = gethostbyaddr ((char *)&client.sin_addr, sizeof(client.sin_addr), AF_INET)) == NULL) {
+      log_err ("gethostbyaddr: %m");
+      return NULL;
+    }
+    else
+	return (strdup (h->h_name));
+  }
+}
+
+/* Traitement pour HyperTerminal */
+
 /* 
- * Convertit la chaine en supprimant les caracteres accentues les
- * plus courants.
+ * Affiche une chaine ISO accentuee (conversion en Videotex)
  */
-static void write_chaine_sans_accent (s)
+static void write_chaine_accentuee (s)
 char *s;
 {
     register unsigned char *p = s;
     char c;
 
     while (*p) {
-	if (*p >= 0xe0 && *p <= 0xe5)
-	    c = 'a';
-	if (*p >= 0xe8 && *p <= 0xeb )
-	   c = 'e';
-	else if (*p >= 0xec && *p <= 0xef)
-	    c = 'i';
-	else if (*p >= 0xf2 && *p <= 0xf6)
-	    c = 'o';
-	else if (*p >= 0xf9 && *p <= 0xf9)
-	    c = 'o';
-	else if (*p == 0xe7)
-	    c = 'c';
-	else
-	    c = *p;
+      if (*p > 192) {
+	int i = *p - 192;
+	write (XTELD_OUTPUT, special_char_table[i], strlen(special_char_table[i]));
+      }
+      else
+	write (XTELD_OUTPUT, p, 1);
 
-	p++;
-	write (XTELD_OUTPUT, &c, 1);
+      p++;
     }
 }	
 
-static void trop_tard ()
+/* Repositionne le curseur a la position courante */
+static void set_cursor (int index) 
 {
-#ifdef USE_SYSLOG
-    log_err ("Pas de reponse du client.\n");
-#else
-    fprintf (fp_console, "Pas de reponse du client.\n");
-#endif /* USE_SYSLOG */
-    exit (1);
+  char str[256];
+
+  sprintf (str, "\037U%c", first_col + index);
+  write (XTELD_OUTPUT, str, strlen (str));
 }
 
-/* La meme chose que "lire_chaine" appelee au lancement de xteld */
-int lire_initial(ch)
+/* Affiche un message a la ligne 0 */
+static void display_control_line (char *s)
+{
+  char str[256];
+
+#ifdef DEBUG_XTELD
+  log_debug ("display_control_line: %s", s);
+#endif
+  sprintf (str, "\024\037@A\030%s\021", s);
+  write (XTELD_OUTPUT, str, strlen (str));
+  erase_needed = 1;
+}
+
+/* Efface la ligne 0 */
+static void erase_control_line (int index)
+{
+  char str[256];
+
+#ifdef DEBUG_XTELD
+  log_debug ("erase_control_line: erase_needed = %d", erase_needed);
+#endif
+  if (erase_needed) {
+    strcpy (str, "\024\037@A\030\021");
+    write (XTELD_OUTPUT, str, strlen (str));
+    if (index >= 0)
+      set_cursor (index);
+
+    erase_needed = 0;
+  }
+}
+
+/* Lecture des services autorises */
+static void read_services_hyper ()
+{
+  register int i, j;
+
+  j = 0;
+  for (i = 0 ; i != nb_services ; i++) {
+      if (service_autorise (i, &temps_maxi)) {
+	services_hyper[j].nom_service = definition_services[i].nom_service;
+	services_hyper[j].nom_uucp = definition_services[i].nom_uucp;
+#ifdef DEBUG_XTELD
+  log_debug ("%d %s %s", j, services_hyper[j].nom_service, services_hyper[j].nom_uucp);
+#endif
+
+	j++;
+      }
+  }
+
+  nb_services_hyper = j;
+#ifdef DEBUG_XTELD
+  log_debug ("%d/%d services autorises", nb_services_hyper, nb_services);
+#endif
+}
+
+/* Affiche un page de services disponibles */
+static void display_page (int start)
+{
+  register int i, ii, istart;
+
+  istart = start * 13;
+
+  if (istart > nb_services_hyper)
+    return;
+
+  /* Efface la page precedente */
+  for (ii = 0 ; ii != 13 ; ii++) {
+    sprintf (buf, "\037%cB\033P\030", ii + 'F');
+    write_chaine_accentuee (buf);
+  }
+
+  /* Affiche la nouvelle */
+  for (ii = 0 ; ii != 13 ; ii++) {
+    i = ii + istart;
+    if (i < nb_services_hyper) {
+      sprintf (buf, "\037%cB\033R\033D %2d\033F\033P %s\n", ii + 'F', i + 1, services_hyper[i].nom_service);
+      write_chaine_accentuee (buf);
+    }
+    else
+      break;
+  }
+}
+
+/* Saisie dans l'ecran Videotex */
+static void erase_input (char *str, int size)
+{
+  memset (str, 0, size);
+  set_cursor (0);
+  while (size--)
+    write (XTELD_OUTPUT, ".", 1);
+  set_cursor (0);
+}
+
+static int input_value (char *str, int max_value, int max_size)
+{
+  char c;
+  int ind = 0;
+
+  if (max_size > IN_SIZE)
+    max_size = IN_SIZE;
+
+  erase_input (str, max_size);
+  
+  do {
+    if (read (XTELD_INPUT, &c, 1) <0)
+      return -1;
+
+#ifdef DEBUG_XTELD
+    log_debug ("read ==> %x", c);
+#endif
+
+    erase_control_line (ind);
+
+    /* Traitement ENVOI/SUITE/RETOUR.. */
+    if (c == 19) {
+      read (XTELD_INPUT, &c, 1);
+
+      switch (c) {
+
+      case 'A' : /* ENVOI */
+
+	if (max_value && (atoi (str) > max_value)) {
+	  display_control_line ("VALEUR INTERDITE !");
+	  erase_input (str, max_size);
+	  ind = 0;
+	}
+	else
+	  c = '\r';
+
+	break;
+
+      case 'B' : /* RETOUR */
+	if (current_page > 0) 
+	  display_page (--current_page);
+	else
+	  display_control_line ("PREMIERE PAGE");
+	set_cursor (ind);
+
+	break;
+
+
+      case 'H' : /* SUITE */
+	if ((current_page  + 1) * 13 > nb_services)
+	  display_control_line ("DERNIERE PAGE");
+	else
+	  display_page (++current_page);
+
+	set_cursor (ind);
+
+	break;
+
+      case 'G' : /* CORRECTION */
+	if (ind > 0) {
+	  str[--ind] = 0;
+	  set_cursor (ind);
+	  write (XTELD_OUTPUT, ".", 1);
+	  set_cursor (ind);
+	}
+
+	break;
+
+      case 'E' : /* ANNULATION */
+
+	erase_input (str, max_size);
+	ind = 0;
+	break;
+
+
+      default :
+	break;
+      }
+    }
+    else {
+      if (ind < max_size && c != '\r') {
+	str[ind++] = c;
+	write (XTELD_OUTPUT, &c, 1);
+#ifdef DEBUG_XTELD
+	log_debug ("str = \"%s\"", str);
+#endif
+      }
+    }
+  } while (c != '\r');
+
+  return (strlen (str));
+}
+
+/* Init du mode HyperTerminal */
+int lire_initial_hyper (ch)
 char *ch;
 {
-    int ret,indice_service;
-    char *ptr;
-
-    /* Demande d'identification */
-    signal (SIGALRM, trop_tard);
-    alarm (20);
-    write (XTELD_OUTPUT, "\x1B\x39\x7b", 3);
-    read (XTELD_INPUT, service, 7);
-    signal (SIGALRM, SIG_IGN);
-
-    if ((!strncmp (service, "\x01SX1",4)) && service[6] == 4) {
-	/* SX1x ==> Emulateur Windows */
-	sprintf (buf, "\x1B\x39\x7A%c%c\x04", tablo_windaube[service[4]], tablo_windaube[service[5]]);
-	write (XTELD_OUTPUT, buf, 6);
-#ifdef DEBUG_XTELD
-	log_debug ("Emulateur Windows, repond : %s", service);
-	log_debug ("RAM = %s", buf);
-#endif
-	write (XTELD_OUTPUT, "\n\r Service : \r\n", 15);
-	/* Pour l'instant, on definit un seul utilisateur Window$: "windaube" */
-	strcpy (ch, "windaube");
-
-	for (indice_service=0; indice_service != nb_services; indice_service++) {
-	    if (service_autorise (indice_service, &temps_maxi)) {
-		sprintf (buf, "%2d - %s\r\n", indice_service, definition_services[indice_service].nom_service);
-		write_chaine_sans_accent (buf);
-	    }
-	}
-	write (XTELD_OUTPUT, "Choisissez un numero : ", 23);
-		
-	ptr = buf-1;
-	do {
-	    ptr++;
-	    ret = read (XTELD_INPUT, ptr, 1);
-#ifdef DEBUG_XTELD
-	    log_debug ("lu : %d", *ptr);
-#endif
-	    if (ret < 0 || ptr-buf > 5) goto err;
-	    write (XTELD_OUTPUT, ptr,1);
-	} while (*ptr != '\r');
-	*ptr = '\000';
+  int ret, iserv, ind = 0;
+  char c, str_in[IN_SIZE+1];
 
 #ifdef DEBUG_XTELD
-	log_debug ("lu : %s",buf);
+  log_debug ("lire_initial_hyper");
 #endif
-	if (atoi(buf) > nb_services || ! service_autorise (atoi(buf), &temps_maxi)) {
-	    sprintf (buf, "\r\n\r\n ** Choix interdit ! **\r\n");
-	    write (XTELD_OUTPUT, buf, strlen(buf));
-	    goto err;
-	}
 
-	/* Il semble que cela soit ncessaire... */
-	write (XTELD_OUTPUT, "\x1B[?31l\x0A\x1B\x3A\x32\x7E\x0A", 12);
-	write (XTELD_OUTPUT, "\x1B\x5B\x3F\x33\x32\x68", 6);
-	write (XTELD_OUTPUT, "\x1B\x5B\x3F\x33\x33\x68", 6);
+  memset ((void*)str_in, 0, sizeof(str_in));
+  current_page = 0;
+  
+  /* Nom du PC connecte */
+  if (peer_name)
+    strcpy (ch , peer_name);
+  else
+    strcpy (ch, "hyper");
 
-	/* nom du service */
-	strcpy (service, definition_services[atoi(buf)].nom_uucp);
+#ifdef DEBUG_XTELD
+  log_debug ("lire_initial_hyper, utilisateur= %s", ch);
+#endif
 
-	return 1;
-    }
+  first_col = 'O';
 
-    return 0;
+  /* Envoi des codes Videotex de la page */
+  sprintf (buf, INIT1_HYPER, version_xtel, revision_xtel, PATCHLEVEL);
+  write (XTELD_OUTPUT, buf, strlen(buf));
+
+  /* Lecture et affichage des services disponibles */
+  read_services_hyper ();
+  display_page (current_page);
+
+  write (XTELD_OUTPUT, INIT2_HYPER, strlen(INIT2_HYPER));
+  erase_control_line (ind);
+    
+  /* Saisie utilisateur */
+  if (input_value (str_in, nb_services_hyper, 2) < 0)
+    goto err;
+
+  iserv = atoi(str_in)-1;
+
+  /* Appel direct, on saisit le numero de telephone */
+  if (!strcmp (services_hyper[iserv].nom_service, "Direct")) {
+    write (XTELD_OUTPUT, INIT3_HYPER, strlen(INIT3_HYPER));
+    first_col = 'J';
+    if (input_value (str_in, 0, 20) < 0)
+      goto err;
+    strcpy (service, str_in);
+    sprintf (buf, "APPEL DIRECT %s", service);
+  }
+
+  /* Appel normal */
+  else {
+    sprintf (service, "\001%s", services_hyper[iserv].nom_service);
+    sprintf (buf, "APPEL %s", services_hyper[iserv].nom_uucp);
+  }
+
+  display_control_line (buf);
+  return 1;
 
 err:
-#ifdef USE_SYSLOG
-    log_err ("read: %m");
-#else
-    fprintf (fp_console, "read: %s\n", sys_errlist[errno]);
-#endif /* USE_SYSLOG */
-    return -1;
+  log_err ("read: %m");
+
+  return -1;
 }
 
 
+
 /*
  *	Partie principale 
  */
@@ -912,6 +1210,7 @@ int ac;
 char **av;
 {
     unsigned char c;
+    char *cp, *progname;
     register int i;
 #ifdef NO_NETWORK
     int sock_ecoute, retry;
@@ -921,29 +1220,41 @@ char **av;
 #endif /* NO_NETWORK */
     int indice_service = 0;
 
-#ifdef USE_SYSLOG
-    openlog("xteld", LOG_PID | LOG_CONS, LOG_DAEMON);
-#else
-    if ((fp_console = fopen ("/dev/console", "w")) == NULL) {
-        perror ("/dev/console");
-        exit (1);
-    }
-#endif /* USE_SYSLOG */
+    progname = xtel_basename (av[0]);
+    init_debug (progname);
 
-    i = 1;
-    while (i < ac) {
-	/* Option support Windaube */
-	if (strcmp (av[i], "-w") == 0) {
+    signal (SIGSEGV, sigsegv);
+
+    /* Lecture de la ligne de commande */
+    while (--ac) {
+	if ((cp = *++av) == NULL)
+	  break;
+	if (*cp == '-' && *++cp) {
+	  switch(*cp) {
+
+	  case 'H' :
 #ifdef DEBUG_XTELD
-	    log_debug ("Option Windaube active !");
+	    log_debug ("Option Windaube/HyperTerminal active !");
 #endif
-	    flag_windaube = True;
-	}
+	    type_client = CLIENT_W_HYPER;
+	    break;
+		      
+	  case 'v' : /* Version */
 
-	i++;
+	    fprintf (stderr, "%s for xtel version %d.%d.%d\n", progname, version_xtel, revision_xtel, PATCHLEVEL);
+	    exit (0);
+	    
+	  default:
+	    usage (progname);
+	    break;
+	  }
+	}
+	else {
+	    break;
+	}
     }
 
-    if ((nb_lignes = lecture_configuration_lignes ()) <= 0) {
+    if ((nb_lignes = lecture_configuration_lignes ()) < 0) {
 	exit (1);
     }
 
@@ -956,11 +1267,7 @@ char **av;
     /* Test de l'existence de /tmp/.xtel */
     if (stat (XTEL_UNIX_PATH, &statb) == 0) {
 	if (unlink (XTEL_UNIX_PATH) < 0) {
-#ifdef USE_SYSLOG
 	    log_err ("unlink: %m");
-#else
-	    fprintf (fp_console, "unlink: %s\n", sys_errlist[errno]);
-#endif /* USE_SYSLOG */
 	    exit (1);
 	}
     }
@@ -999,11 +1306,7 @@ char **av;
      */
 
     if ((sock_ecoute = socket (AF_UNIX, SOCK_STREAM, 0))  < 0) {
-#ifdef USE_SYSLOG
 	log_err("socket: %m");
-#else
-	fprintf (fp_console, "socket: %s\n", sys_errlist[errno]);
-#endif /* USE_SYSLOG */
 	exit (1);
     }
 
@@ -1012,22 +1315,13 @@ char **av;
     strcpy (unaddr.sun_path, XTEL_UNIX_PATH);
 
     if (bind (sock_ecoute, (struct sockaddr *)&unaddr, sizeof(unaddr)) < 0) {
-#ifdef USE_SYSLOG
 	log_err ("bind: %m");
-#else
-	fprintf (fp_console, "bind: %s\n", sys_errlist[errno]);
-#endif /* USE_SYSLOG */
 	exit (1);
     }
 
     /* Ouvre le service */
     if (listen (sock_ecoute, 5) < 0) {
-#ifdef USE_SYSLOG
       log_err ("listen: %m");
-#else
-      fprintf (fp_console, "listen: %s\n", sys_errlist[errno]);
-#endif /* USE_SYSLOG */
-
       exit (1);
     }
 
@@ -1050,11 +1344,7 @@ char **av;
 		break;
 	    }
 	    else if (!retry) {
-#ifdef USE_SYSLOG
 		log_err ("accept: %m");
-#else
-		fprintf (fp_console, "accept: %s\n", sys_errlist[errno]);
-#endif /* USE_SYSLOG */
 		exit (1);
 	    }
 	}
@@ -1062,9 +1352,10 @@ char **av;
 	/*
 	 * Cree le processus de service (fils)
 	 */
-
 	if (!fork()) {
-
+#else
+	  /* Lecture du nom du client connecte */
+	  peer_name = get_peer_name ();
 #endif /* NO_NETWORK */
 
 #ifdef DEBUG_XTELD
@@ -1072,15 +1363,24 @@ char **av;
 #endif
 
 	    /* Lecture du nom d'utilisateur */
-	    if (flag_windaube) {
-		if (lire_initial(utilisateur) < 0)
+	    switch (type_client) {
+
+	    case CLIENT_W_HYPER:
+		if (lire_initial_hyper (utilisateur) < 0)
 		    goto fin_xteld;
-		write (XTELD_OUTPUT, "\x1B\x54\x20\x0F Appel...", 13);
+
 		appel_service (service);
-	    }
-	    else {
-		if (lire_chaine(utilisateur) < 0)
+		break;
+
+	    case CLIENT_XTEL :
+		if (lire_chaine (utilisateur) < 0)
 		    goto fin_xteld;
+		
+		break;
+
+	    default :
+	      goto fin_xteld;
+		break;
 	    }
 
 	    /* lecture connexion XTEL */
@@ -1217,7 +1517,5 @@ fin_xteld:
 
 #endif /* NO_NETWORK */
 
-#ifndef USE_SYSLOG
-    fclose (fp_console);
-#endif /* NO_NETWORK */
+    close_debug ();
 }
